from .compressor import Compressor

__version__           = Compressor.__version__
__short_description__ = "Storer Tool"
__license__           = "MIT"
__author__            = "Alexander D. Kazakov"
__author_email__      = "alexander.d.kazakov@gmail.com"
__github_username__   = "AlexanderDKazakov"