# -*- coding: utf-8 -*-
from .storer import Storer

__all__ = (
    '__version__',
    'Storer'
)

__version__           = Storer.__version__
__short_description__ = "Minimalist storage class for any purpose."
__license__           = "MIT"
__author__            = "Alexander D. Kazakov"
__author_email__      = "alexander.d.kazakov@gmail.com"
__github_username__   = "AlexanderDKazakov"
